@@ -1,3 +1,5 @@
+/* $XdotOrg: driver/xf86-video-nv/src/nv_driver.c,v 1.21 2006/01/24 16:45:29 aplattner Exp $ */
+/* $XConsortium: nv_driver.c /main/3 1996/10/28 05:13:37 kaleb $ */
 /*
  * Copyright 1996-1997  David J. McKay
  *
@@ -27,10 +29,11 @@
 /* Hacked together from mga driver and 3.3.4 NVIDIA driver by Jarno Paananen
    <jpaana@s2.org> */
 
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nv/nv_driver.c,v 1.144 2006/06/16 00:19:32 mvojkovi Exp $ */
+
 #include "nv_include.h"
 
 #include "xf86int10.h"
-#include "vbeModes.h"
 
 const   OptionInfoRec * RivaAvailableOptions(int chipid, int busid);
 Bool    RivaGetScrnInfoRec(PciChipsets *chips, int chip);
@@ -67,10 +70,8 @@ static Bool	NVMapMem(ScrnInfoPtr pScrn);
 static Bool	NVMapMemFBDev(ScrnInfoPtr pScrn);
 static Bool	NVUnmapMem(ScrnInfoPtr pScrn);
 static void	NVSave(ScrnInfoPtr pScrn);
-static void	NVSaveRestoreVBE(ScrnInfoPtr, vbeSaveRestoreFunction);
 static void	NVRestore(ScrnInfoPtr pScrn);
 static Bool	NVModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
-static Bool	NVSetModeVBE(ScrnInfoPtr pScrn, DisplayModePtr pMode);
 
 
 /*
@@ -91,7 +92,7 @@ _X_EXPORT DriverRec NV = {
         0
 };
 
-/* Known cards as of 2008/01/24 */
+/* Known cards as of 2006/06/16 */
 
 static SymTabRec NVKnownChipsets[] =
 {
@@ -332,7 +333,7 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0244, "GeForce Go 6150" },
   { 0x10DE0247, "GeForce Go 6100" },
 
-#if SUPPORT_G80
+/*************** G8x ***************/
   { 0x10DE0191, "GeForce 8800 GTX" },
   { 0x10DE0193, "GeForce 8800 GTS" },
   { 0x10DE0194, "GeForce 8800 Ultra" },
@@ -340,30 +341,9 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE019E, "Quadro FX 4600" },
   { 0x10DE0400, "GeForce 8600 GTS" },
   { 0x10DE0402, "GeForce 8600 GT" },
-  { 0x10DE0404, "GeForce 8400 GS" },
-  { 0x10DE0407, "GeForce 8600M GT" },
-  { 0x10DE0409, "GeForce 8700M GT" },
-  { 0x10DE040A, "Quadro FX 370" },
-  { 0x10DE040B, "Quadro NVS 320M" },
-  { 0x10DE040C, "Quadro FX 570M" },
-  { 0x10DE040D, "Quadro FX 1600M" },
-  { 0x10DE040E, "Quadro FX 570" },
-  { 0x10DE040F, "Quadro FX 1700" },
   { 0x10DE0421, "GeForce 8500 GT" },
   { 0x10DE0422, "GeForce 8400 GS" },
   { 0x10DE0423, "GeForce 8300 GS" },
-  { 0x10DE0425, "GeForce 8600M GS" },
-  { 0x10DE0426, "GeForce 8400M GT" },
-  { 0x10DE0427, "GeForce 8400M GS" },
-  { 0x10DE0428, "GeForce 8400M G" },
-  { 0x10DE0429, "Quadro NVS 140M" },
-  { 0x10DE042A, "Quadro NVS 130M" },
-  { 0x10DE042B, "Quadro NVS 135M" },
-  { 0x10DE042D, "Quadro FX 360M" },
-  { 0x10DE042F, "Quadro NVS 290" },
-  { 0x10DE0611, "GeForce 8800 GT" },
-  { 0x10DE061A, "Quadro FX 3700" },
-#endif
 
   {-1, NULL}
 };
@@ -430,17 +410,6 @@ static const char *vbeSymbols[] = {
     "vbeDoEDID",
     NULL
 };
-
-static const char *vbeModeSymbols[] = {
-    "VBEExtendedInit",
-    "VBEGetVBEInfo",
-    "VBEGetModePool",
-    "VBEValidateModes",
-    "VBESetModeParameters",
-    "VBEGetVBEMode",
-    "VBESetVBEMode",
-    NULL
-};
 #endif
 
 static const char *i2cSymbols[] = {
@@ -519,8 +488,7 @@ typedef enum {
     OPTION_FP_DITHER,
     OPTION_CRTC_NUMBER,
     OPTION_FP_SCALE,
-    OPTION_FP_TWEAK,
-    OPTION_DUALHEAD,
+    OPTION_FP_TWEAK
 } NVOpts;
 
 
@@ -537,7 +505,6 @@ static const OptionInfoRec NVOptions[] = {
     { OPTION_CRTC_NUMBER,	"CrtcNumber",	OPTV_INTEGER,	{0}, FALSE },
     { OPTION_FP_SCALE,          "FPScale",      OPTV_BOOLEAN,   {0}, FALSE },
     { OPTION_FP_TWEAK,          "FPTweak",      OPTV_INTEGER,   {0}, FALSE },
-    { OPTION_DUALHEAD,          "DualHead",     OPTV_BOOLEAN,   {0}, FALSE },
     { -1,                       NULL,           OPTV_NONE,      {0}, FALSE }
 };
 
@@ -693,7 +660,6 @@ NVGetPCIXpressChip (pciVideoPtr pVideo)
     return pciid;
 }
 
-#if SUPPORT_G80
 static Bool
 NVIsG80(int chipType)
 {
@@ -701,16 +667,11 @@ NVIsG80(int chipType)
         case 0x0190:
         case 0x0400:
         case 0x0420:
-        case 0x0610:
-        case 0x0620:
-        case 0x0630:
-        case 0x0640:
             return TRUE;
     }
 
     return FALSE;
 }
-#endif
 
 /* Mandatory */
 static Bool
@@ -796,10 +757,8 @@ NVProbe(DriverPtr drv, int flags)
                default:  break;  /* we don't recognize it */
                }
 
-#if SUPPORT_G80
                if(NVIsG80((*ppPci)->chipType))
                    canHandle = TRUE;
-#endif
 
                if(canHandle) {
                    NVChipsets[numUsed].token = pciid;
@@ -837,11 +796,9 @@ NVProbe(DriverPtr drv, int flags)
         if(pPci->vendor == PCI_VENDOR_NVIDIA_SGS) {
             if(RivaGetScrnInfoRec(NVPciChipsets, usedChips[i]))
                 foundScreen = TRUE;
-#if SUPPORT_G80
         } else if (NVIsG80(pPci->chipType)) {
             if(G80GetScrnInfoRec(NVPciChipsets, usedChips[i]))
                 foundScreen = TRUE;
-#endif
         } else {
             if(NVGetScrnInfoRec(NVPciChipsets, usedChips[i])) 
 	        foundScreen = TRUE;
@@ -864,27 +821,6 @@ NVSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
     return NVModeInit(pScrn, mode);
 }
 
-Bool
-NVSwitchModeVBE(int scrnIndex, DisplayModePtr mode, int flags)
-{
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
-    NVPtr pNv = NVPTR(pScrn);
-    const Bool disableAccess = pNv->accessEnabled;
-
-    if(disableAccess)
-        pScrn->EnableDisableFBAccess(scrnIndex, FALSE);
-
-    NVSync(pScrn);
-    if (!NVSetModeVBE(pScrn, mode))
-        return FALSE;
-    NVAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
-
-    if(disableAccess)
-        pScrn->EnableDisableFBAccess(scrnIndex, TRUE);
-
-    return TRUE;
-}
-
 /*
  * This function is used to initialize the Start Address - the first
  * displayed location in the video memory.
@@ -933,17 +869,6 @@ NVEnterVTFBDev(int scrnIndex, int flags)
     return TRUE;
 }
 
-static Bool
-NVEnterVTVBE(int scrnIndex, int flags)
-{
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
-
-    if (!NVSetModeVBE(pScrn, pScrn->currentMode))
-        return FALSE;
-    NVAdjustFrame(scrnIndex, 0, 0, 0);
-    return TRUE;
-}
-
 /*
  * This is called when VT switching away from the X server.  Its job is
  * to restore the previous (text) mode.
@@ -963,14 +888,7 @@ NVLeaveVT(int scrnIndex, int flags)
     NVLockUnlock(pNv, 1);
 }
 
-static void
-NVLeaveVTVBE(int scrnIndex, int flags)
-{
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
 
-    NVSync(pScrn);
-    NVSaveRestoreVBE(pScrn, MODE_RESTORE);
-}
 
 static void 
 NVBlockHandler (
@@ -1012,15 +930,9 @@ NVCloseScreen(int scrnIndex, ScreenPtr pScreen)
     NVPtr pNv = NVPTR(pScrn);
 
     if (pScrn->vtSema) {
-        if (!pNv->NoAccel)
-            NVSync(pScrn);
-
-        if (pNv->VBEDualhead) {
-            NVSaveRestoreVBE(pScrn, MODE_RESTORE);
-        } else {
-            NVRestore(pScrn);
-            NVLockUnlock(pNv, 1);
-        }
+        NVSync(pScrn);
+        NVRestore(pScrn);
+        NVLockUnlock(pNv, 1);
     }
 
     NVUnmapMem(pScrn);
@@ -1044,16 +956,6 @@ NVCloseScreen(int scrnIndex, ScreenPtr pScreen)
     return (*pScreen->CloseScreen)(scrnIndex, pScreen);
 }
 
-static void
-NVEnableDisableFBAccess(int scrnIndex, Bool enable)
-{
-    NVPtr pNv = NVPTR(xf86Screens[scrnIndex]);
-
-    pNv->accessEnabled = enable;
-    pNv->EnableDisableFBAccess(scrnIndex, enable);
-}
-
-
 /* Free up any persistent data structures */
 
 /* Optional */
@@ -1119,7 +1021,6 @@ Bool NVI2CInit(ScrnInfoPtr pScrn)
 }
 
 
-#ifdef USE_CVTMODE_FUNC
 /* Copied from ddc/Property.c */
 static DisplayModePtr
 NVModesAdd(DisplayModePtr Modes, DisplayModePtr Additions)
@@ -1143,7 +1044,6 @@ NVModesAdd(DisplayModePtr Modes, DisplayModePtr Additions)
 
     return Modes;
 }
-#endif
 
 /* Mandatory */
 Bool
@@ -1154,9 +1054,7 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     int i, max_width, max_height;
     ClockRangePtr clockRanges;
     const char *s;
-#ifdef USE_CVTMODE_FUNC
-    int config_mon_rates;
-#endif
+    Bool config_mon_rates;
 
     if (flags & PROBE_DETECT) {
         EntityInfoPtr pEnt = xf86GetEntityInfo(pScrn->entityList[0]);
@@ -1506,43 +1404,7 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     } else {
         pNv->usePanelTweak = FALSE;
     }
-
-    if (xf86ReturnOptValBool(pNv->Options, OPTION_DUALHEAD, FALSE)) {
-        if (pNv->FBDev)
-            xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
-                       "FBDev and Dualhead are incompatible.\n");
-        else
-            pNv->VBEDualhead = TRUE;
-    }
-
-    if (pNv->VBEDualhead) {
-        if (!xf86LoadSubModule(pScrn, "vbe")) {
-            xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-                       "Couldn't load the VBE module and Dualhead is "
-                       "enabled.\n");
-            return FALSE;
-        }
-        xf86LoaderReqSymLists(vbeModeSymbols, NULL);
-        pNv->pVbe = VBEExtendedInit(NULL, pNv->pEnt->index,
-                                    SET_BIOS_SCRATCH | RESTORE_BIOS_SCRATCH);
-        if (!pNv->pVbe) return FALSE;
-
-        pNv->pVbeInfo = VBEGetVBEInfo(pNv->pVbe);
-        if (!pNv->pVbeInfo) return FALSE;
-
-        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG,
-                   "Using VBE dual-head mode.\n");
-
-        xf86DrvMsg(pScrn->scrnIndex, X_INFO,
-                   "Using software cursor.\n");
-        pNv->HWCursor = FALSE;
-
-        pScrn->SwitchMode    = NVSwitchModeVBE;
-        pScrn->EnterVT       = NVEnterVTVBE;
-        pScrn->LeaveVT       = NVLeaveVTVBE;
-        pScrn->ValidMode     = NULL;
-    }
-
+    
     if (pNv->pEnt->device->MemBase != 0) {
 	/* Require that the config file value matches one of the PCI values. */
 	if (!xf86CheckPciMemBase(pNv->PciInfo, pNv->pEnt->device->MemBase)) {
@@ -1650,13 +1512,11 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     pNv->alphaCursor = (pNv->Architecture >= NV_ARCH_10) &&
                        ((pNv->Chipset & 0x0ff0) != 0x0100);
 
-#ifdef USE_CVTMODE_FUNC
     if ((pScrn->monitor->nHsync == 0) && 
 	(pScrn->monitor->nVrefresh == 0))
 	config_mon_rates = FALSE;
     else
 	config_mon_rates = TRUE;
-#endif
 
     NVCommonSetup(pScrn);
 
@@ -1727,7 +1587,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
        max_height = 4096;
     }
 
-#ifdef USE_CVTMODE_FUNC
     /* If DFP, add a modeline corresponding to its panel size */
     if (pNv->FlatPanel && !pNv->Television && pNv->fpWidth && pNv->fpHeight) {
 	DisplayModePtr Mode;
@@ -1756,7 +1615,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	    pScrn->monitor->nVrefresh = 1;
 	}
     }
-#endif
 
     /*
      * xf86ValidateModes will check that the mode HTotal and VTotal values
@@ -1764,31 +1622,14 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
      * pScrn->maxVValue are set.  Since our NVValidMode() already takes
      * care of this, we don't worry about setting them here.
      */
-    if (pNv->VBEDualhead) {
-        pScrn->modePool = VBEGetModePool(pScrn, pNv->pVbe, pNv->pVbeInfo,
-                                         V_MODETYPE_VBE);
-
-        VBESetModeNames(pScrn->modePool);
-        i = VBEValidateModes(pScrn, pScrn->monitor->Modes,
-                             pScrn->display->modes, clockRanges,
-                             NULL, 256, max_width,
-                             512, 128, max_height,
-                             pScrn->display->virtualX,
-                             pScrn->display->virtualY,
-                             pNv->ScratchBufferStart,
-                             LOOKUP_BEST_REFRESH);
-        if (i > 0)
-            VBESetModeParameters(pScrn, pNv->pVbe);
-    } else {
-        i = xf86ValidateModes(pScrn, pScrn->monitor->Modes,
-                              pScrn->display->modes, clockRanges,
-                              NULL, 256, max_width,
-                              512, 128, max_height,
-                              pScrn->display->virtualX,
-                              pScrn->display->virtualY,
-                              pNv->ScratchBufferStart,
-                              LOOKUP_BEST_REFRESH);
-    }
+    i = xf86ValidateModes(pScrn, pScrn->monitor->Modes,
+                          pScrn->display->modes, clockRanges,
+                          NULL, 256, max_width,
+                          512, 128, max_height,
+                          pScrn->display->virtualX,
+                          pScrn->display->virtualY,
+                          pNv->ScratchBufferStart,
+                          LOOKUP_BEST_REFRESH);
 
     if (i < 1 && pNv->FBDev) {
 	fbdevHWUseBuildinMode(pScrn);
@@ -1821,22 +1662,6 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
      */
     xf86SetCrtcForModes(pScrn, 0);
 
-    if (pNv->VBEDualhead) {
-        DisplayModePtr p = pScrn->modes;
-
-        /*
-         * Loop through modes and double their widths.  Stash the real width in
-         * CrtcHDisplay.  Also adjust the screen dimensions.
-         */
-        do {
-            p->CrtcHDisplay = p->HDisplay;
-            p->HDisplay *= 2;
-        } while ((p = p->next) != pScrn->modes);
-
-        pScrn->virtualX *= 2;
-        pScrn->displayWidth *= 2;
-    }
-
     /* Set the current mode to the first in the list */
     pScrn->currentMode = pScrn->modes;
 
@@ -2018,32 +1843,6 @@ NVModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     return TRUE;
 }
 
-static Bool
-NVSetModeVBE(ScrnInfoPtr pScrn, DisplayModePtr pMode)
-{
-    NVPtr pNv = NVPTR(pScrn);
-    VbeModeInfoData *data;
-    int mode;
-
-    data = (VbeModeInfoData*)pMode->Private;
-    mode = data->mode | 1 << 14;
-
-    if(!VBESetVBEMode(pNv->pVbe, mode, data->block))
-        return FALSE;
-    pNv->PCRTC0[0x820/4] = pNv->PCRTC0[0x2820/4] =
-        pScrn->displayWidth * (pScrn->bitsPerPixel / 8);
-    pNv->vbeCRTC1Offset = pMode->CrtcHDisplay * (pScrn->bitsPerPixel / 8);
-
-    pScrn->vtSema = TRUE;
-
-    NVLoadStateExt(pNv, NULL);
-    NVResetGraphics(pScrn);
-
-    pNv->CurrentLayout.mode = pMode;
-
-    return TRUE;
-}
-
 /*
  * Restore the initial (text) mode.
  */
@@ -2055,10 +1854,8 @@ NVRestore(ScrnInfoPtr pScrn)
     NVPtr pNv = NVPTR(pScrn);
     NVRegPtr nvReg = &pNv->SavedReg;
 
-    if(pNv->HWCursor) {
+    if(pNv->HWCursor)
         NVShowHideCursor(pNv, 0);
-        sleep(1);
-    }
     NVLockUnlock(pNv, 0);
 
     if(pNv->twoHeads) {
@@ -2218,10 +2015,6 @@ NVScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	fbdevHWSave(pScrn);
 	if (!fbdevHWModeInit(pScrn, pScrn->currentMode))
 	    return FALSE;
-    } else if (pNv->VBEDualhead) {
-        NVSaveRestoreVBE(pScrn, MODE_SAVE);
-        if (!NVSetModeVBE(pScrn, pScrn->currentMode))
-            return FALSE;
     } else {
 	/* Save the current state */
 	NVSave(pScrn);
@@ -2417,10 +2210,6 @@ NVScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     pNv->BlockHandler = pScreen->BlockHandler;
     pScreen->BlockHandler = NVBlockHandler;
 
-    pNv->accessEnabled = TRUE;
-    pNv->EnableDisableFBAccess = pScrn->EnableDisableFBAccess;
-    pScrn->EnableDisableFBAccess = NVEnableDisableFBAccess;
-
 #ifdef RANDR
     /* Install our DriverFunc.  We have to do it this way instead of using the
      * HaveDriverFuncs argument to xf86AddDriver, because InitOutput clobbers
@@ -2459,20 +2248,6 @@ NVSave(ScrnInfoPtr pScrn)
     NVDACSave(pScrn, vgaReg, nvReg, pNv->Primary);
 }
 
-static void
-NVSaveRestoreVBE(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function)
-{
-    NVPtr pNv = NVPTR(pScrn);
-
-    if (function == MODE_SAVE) {
-        VBEGetVBEMode(pNv->pVbe, &pNv->vbeMode);
-        NVSave(pScrn);
-    } else if (function == MODE_RESTORE) {
-        NVRestore(pScrn);
-        VBESetVBEMode(pNv->pVbe, pNv->vbeMode, NULL);
-    }
-}
-
 #ifdef RANDR
 static Bool
 NVRandRGetInfo(ScrnInfoPtr pScrn, Rotation *rotations)
@@ -2533,4 +2308,4 @@ NVDriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op, pointer data)
 
     return FALSE;
 }
-#endif /* RANDR */
+#endif
